/*
    Biribot.ino
    Release 1.0


  ToDo;
  
    

  Avalaible Actions to be programmed in loop()
      
  Move( FORWARD , V_ACT );
  Move( REVERSE , V_ACT );

  Rotate( RIGHT , V_ACT );
  Rotate( LEFT , V_ACT );

  Turn( LEFT_FW , V_ACT );
  Turn( LEFT_RV , V_ACT );
  Turn( RIGHT_FW , V_ACT );
  Turn( RIGHT_RV , V_ACT );

  LM_Test();
  RM_Test();
  Drivetrain_test();

  PAUSE_1;
  PAUSE_2;
  PAUSE_5;

  FOREVER;

*/




// The following line must be set to
//#define DEBUG or #define RELEASE or #define TEST ( See loop()
// DEBUG uses the Serial Monitor and Imprime() function
#define RELEASE                  

#define FOREVER for(;;)
#define PAUSE_1 delay(1000)
#define PAUSE_2 delay(2000)
#define PAUSE_5 delay(5000)


#define NO                    0
#define SI                  !NO



// Speeds
#define V_MIN                5
#define V_MAX                80
#define V_TEST               25
#define V_ACT                50    


// Directions
#define STOP                  0
#define FORWARD               1
#define REVERSE               2
#define LEFT                  3
#define RIGHT                 4
#define LEFT_FW               5     
#define LEFT_RV               6
#define RIGHT_FW              7
#define RIGHT_RV              8



// Interface with hardware

//  Motors ( Servos)
#define LM_PIN                9    //  Left motor    White, Orange
#define RM_PIN               10    //  Right motor   Yellow


//  HCSR04 Ultrasonic sensor
// Used in Read__distance() HCSR04 function
#define HCSR04_K             58.0    // Changing this value modified the distance read by the sensor
#define HCSR04_TRIG_PIN      11     //  Trigger       Yellow
#define HCSR04_ECHO_PIN      12     //  Echo          Orange
#define HCSR04_PULSEWIDTH    10    //   Trigger pulse width ( mcrosegs )

#define DIST_MAX             15.0
#define DIST_MIN_MAX         10.0     // Biribot will stop when he gets to this distance
#define DIST_MIN_MIN          5.0     // ... and will move forward slower until he gets this distance
#define DIST_ERR              2.0
#define NUM_DEC               1


// Servos center positions ( Should be 90 but what can you expect of a 2 bucks Chinese servo? Center = 90 ?? Hahaha 
#define LM_CENTER            80
#define RM_CENTER            83



/*
    Prototypes
*/
float  Read_distance(void);
void Imprime( float );            //Imprime = Print( distance );


void Engine_start( void );
void Engine_stop( void );


void Move( int , int );
void Turn( int , int );
void Rotate( int , int );
void Neutral( void );
void Change( int , int , int ); // padonde, v_min, v_max
void lm_move( int , int );
void rm_move( int , int );

void LM_Test( void );
void RM_Test( void );
void Drivetrain_test( void );




//   Servos ( drive train servos, left-motor & right-motor )
#include <Servo.h> 
Servo lm,rm;





void setup() 
{ 
  // Ultrasonic HCSR04
  pinMode(HCSR04_TRIG_PIN, OUTPUT);
  pinMode(HCSR04_ECHO_PIN, INPUT);
  digitalWrite(HCSR04_TRIG_PIN, LOW);  // Starts HCSR04 trigger pulse to LOW

  #if defined RELEASE
    Engine_start();
    PAUSE_5;          // Nice to wait for a while b4 moving ...
  #endif
  #if defined DEBUG
    Serial.begin(9600);
  #endif 
} 



/*

    main loop

*/
void loop() 
{ 
  static float ant , pos =0.0;

  //  Drivetrain Test. Change the first #define in the code to #define TEST
  #if defined TEST
    Drivetrain_test();
    FOREVER;                  //   Stop here
  #endif

  //  Debug version. Change the first #define in the code to #define DEBUG
  #if defined DEBUG
    FOREVER
    {
       pos = Read_distance();
       Imprime( pos);
       PAUSE_1;
    }
  #endif
  
  //  Release version. Change the first #define in the code to #define RELEASE
  #if defined RELEASE

    //  0) Go ahead until we get to DIST_MIN_MAX of an object 
    Move( FORWARD , V_ACT );
    do 
    {
        // Feedback code for servos here
    } while( ( pos = Read_distance())  > DIST_MIN_MAX );

      // Avoiding strategy code starts HERE points 1) to 4)
    
    //  1) Go Neutral and stop for a second ...
    Neutral();
    PAUSE_1;
    
    // Aqui debemos ponernos de frente coño !!!
    
    //  2) We go to DIST_MIN_MIN slowly, in a different and funny do-while loop     ...
    for( pos=Read_distance(), ant=pos+0.5 ; pos > DIST_MIN_MIN ; pos= Read_distance() )
    {
      Move( FORWARD , V_MIN);
      delay(15);
      Neutral();      // To Read_distance not moving we stop a bit
              
      // Here we see a distance shorter than before
      if( pos < ant )
      { 
        ant=pos;
        // Imprime(pos); 
      
        // Here we fill the vector of IR sensor measures (In a future post with IR )
      }
      
    }
    
    // 3) ... Move back to DIST_MAX
    Neutral();
    PAUSE_1;
    Move( REVERSE , V_ACT );
    do 
    {
         // Feedback code for servos here
    } while(  (pos = Read_distance() ) < DIST_MAX );
    Neutral();
    
    // 4) ... Turn right until we find enough distance to go forward  and ...
    Neutral();
    PAUSE_1;
    do
     {
       Rotate( RIGHT , V_ACT );
       PAUSE_1;
       Neutral();
       delay(50);
     } while( ( pos = Read_distance()) < DIST_MAX );

    // ... and stop for a second and start again the loop
    Neutral();
    PAUSE_1;
        
  #endif  
} 
/*
    
      End of loop()


   __________________________________________________________________________________________________________________________
*/






/*
    Read_distance( void )

    Notes: In setup() we start the pulse to LOW, and we finish in this function to LOW
           So we don't need to start with digitalWrite( TRIG_PIN, LOW )
*/
float Read_distance(void)
{
  unsigned long pulso;
  int aux;
  
  // Trigger pulse in HCSR04 sensor. We need only a 5/10 microsg pulse
  digitalWrite(HCSR04_TRIG_PIN, HIGH);
  delayMicroseconds(HCSR04_PULSEWIDTH);
  digitalWrite(HCSR04_TRIG_PIN, LOW);
    
  //Read the Echo if exists
  pulso = pulseIn(HCSR04_ECHO_PIN, HIGH);  //pulso = pulseIn(HCSR04_ECHO_PIN, HIGH,1500); 1500 shortens the range to 4 cm - 20 cm

  #if defined DEBUG
    Serial.print( "   Pulso ... ");
    Serial.print( pulso  );
  #endif
  
  aux=  (int) 10*(0.5+pulso/HCSR04_K ) ;
  return aux/10.0; 
}




#if defined DEBUG

/*
    Imprime(( float distancia )
*/
void Imprime ( float distancia )
{
  
  Serial.print( "   Distancia ... ");
  Serial.println( distancia , NUM_DEC );
}

#endif




/*
    Engine_start()
*/
void Engine_start( void )
{
  lm.attach(LM_PIN);    
  rm.attach(RM_PIN);  
  Neutral();
  PAUSE_1;
}



/*
    Engine_stop()
*/
void Engine_stop( void )
{
  Neutral();
  PAUSE_1;
  lm.detach();    
  rm.detach();  
}



/*
    Move()
    
    Move Biribot moving both motors the same direction
    
      pars:
        where: { FORWARD , REVERSE }
        speed : { 0 , V_MAX  }
*/
void Move( int where , int speed )
{
  switch( where )
  {
    case FORWARD:
      lm_move( FORWARD , speed);  //Retoque
      rm_move( FORWARD , speed);
      break;
    case REVERSE:
      lm_move( REVERSE , speed);  //Retoque
      rm_move( REVERSE , speed);
      break;
    default: 
      return;
  }
}



/*
    Turn()
    
    Turn Biribot moving only one motor FORWARD/REVERSE
    
      pars:
        where: { LEFT_FW , LEFT_RV , RIGHT_FW , RIGHT_RV }     ( FW: FORWARD , RV: REVERSE )
        speed : { 0 , V_MAX  }
*/
void Turn( int where , int speed )
{
  switch( where )
  {
    case LEFT_FW:
      lm_move( FORWARD , speed);
      rm_move( STOP , speed);
      break;
    case LEFT_RV:
      lm_move( REVERSE , speed);
      rm_move( STOP , speed);
      break;

    case RIGHT_FW:
      lm_move( STOP , speed);
      rm_move( FORWARD , speed);
      break;
    case RIGHT_RV:
      lm_move( STOP , speed);
      rm_move( REVERSE , speed);
      break;
    default:
      return;
  }
}



/*
    Rotate()
    
    Rotate Biribot moving both motor opposite directions ( ie lm FORWARD , rm REVERSE )
    
      pars:
        where: { LEFT , RIGHT }
        speed : { 0 , V_MAX  }
*/
void Rotate( int where , int speed )
{
  switch( where )
  {
    case LEFT:
      lm_move( FORWARD , speed);
      rm_move( REVERSE , speed);
      break;
    case RIGHT:
      lm_move( REVERSE , speed);
      rm_move( FORWARD , speed);
      break;
    default:
      return;
  }
}



/*
    Neutral()

    Stops motors moving each to center position    
*/
void Neutral(void)
{
  lm.write(LM_CENTER);
  rm.write(RM_CENTER);
}




/*******************************************************************************************************************/





/*
    lm_move()
    
    Move Left Motor 
    
      pars:
        where: { FORWARD , REVERSE , STOP }
        speed : { 0 , V_MAX  }
*/
void lm_move( int where , int speed )
{
  if( (speed > V_MAX) || (speed < V_MIN) )
    return;

  switch( where )
  {
    case FORWARD:
      lm.write(LM_CENTER+speed); 
      break;
    case REVERSE:
      lm.write(LM_CENTER-speed); 
      break;
    case STOP:
    default:
      lm.write(LM_CENTER);
  }
}



/*
    rm_move()

    Move Right Motor 
    
      pars:
        where: { FORWARD , REVERSE , STOP }
        speed : { 0 , V_MAX  }
*/
void rm_move( int where , int speed )
{
  if( (speed > V_MAX) || (speed < V_MIN) )
    return;

  switch( where )
  {
    case FORWARD:
      rm.write(RM_CENTER-speed);
      break;
    case REVERSE:
      rm.write(RM_CENTER+speed);
      break;
    case STOP:
    default:
      rm.write(RM_CENTER);
  }
}



/*
    Left Motor Test
*/
void LM_Test( void )
{
  lm_move( FORWARD , V_TEST );
  PAUSE_2;
  lm_move( STOP , V_TEST );
  PAUSE_2;
  lm_move( REVERSE , V_TEST );
  PAUSE_2;
  lm_move( STOP , V_TEST );
  PAUSE_2;
}



/*
    Right Motor Test
*/
void RM_Test( void )
{
  rm_move( FORWARD , V_TEST );
  PAUSE_2;
  rm_move( STOP , V_TEST );
  PAUSE_2;
  rm_move( REVERSE , V_TEST );
  PAUSE_2;
  rm_move( STOP , V_TEST );
  PAUSE_2;
}



/*
    Right Motor Test
*/
void Drivetrain_test( void )
{
  Move( FORWARD , V_ACT );
  PAUSE_5;
  Neutral();
  PAUSE_2;
  Move( REVERSE , V_ACT );
  PAUSE_5;
  Neutral();
  PAUSE_2;

  Rotate( RIGHT , V_ACT );
  PAUSE_2;
  Neutral();
  PAUSE_2;

  Rotate( LEFT , V_ACT );
  PAUSE_2;
  Neutral();
  PAUSE_2;

  Turn( LEFT_FW , V_ACT );
  PAUSE_2;
  Neutral();
  PAUSE_2;

  Turn( LEFT_RV , V_ACT );
  PAUSE_2;  
  Neutral();
  PAUSE_2;
  
  Turn( RIGHT_FW , V_ACT );
  PAUSE_2;  
  Neutral();
  PAUSE_2;
    
  Turn( RIGHT_RV , V_ACT );
  PAUSE_2;      
  Neutral();
  PAUSE_2;
}      


